'use strict';

let createApplicationAsync = (() => {
  var _ref = _asyncToGenerator(function* (app) {
    let config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    try {
      if (!app) {
        app = express();
        config = require('./config');
        exports.config = config;
      }
      exports.app = app;

      logger.initialize(config.logger);
      const server = http.createServer(app);

      var _ref2 = yield load(config.swagger, exports);

      const api = _ref2.api,
            middlewares = _ref2.middlewares,
            models = _ref2.models;


      exports.createTestApplicationAsync = function () {
        return createTestApplicationAsync(app, api, config);
      };
      exports.getRestMiddleware = exports.restMiddleware = function () {
        return getRestMiddleware({ api, middlewares, errorHandler: config.errorHandler });
      };

      if (config.model) {
        yield createModelAsync(config.model, models, exports);
      }

      if (config.session) {
        app.use(createSession(config.session));
      }

      if (config.ws) {
        const wsapp = createWebSocketApplication(server, api, config.ws);
        app.wsapp = wsapp;
        exports.wsapp = wsapp;
        app.use(wsapp.extendMiddleware);
        // TODO: find a better way to mount handle in the end
        setImmediate(function () {
          wsapp.use(exports.getRestMiddleware());
        });
      }

      app.listen = function listen() {
        return server.listen.apply(server, arguments);
      };

      return app;
    } catch (err) {
      console.log(err.stack); // eslint-disable-line
      process.exit(1);
    }
  });

  return function createApplicationAsync(_x) {
    return _ref.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const http = require('http');

const express = require('express');
const Router = require('router');
const Sequelize = require('sequelize');

const rc = require('./rc');
const utility = require('./utility');
const Pipeline = require('./pipeline');
const mws = require('./middleware');
const logger = require('./logger');
const load = require('./load');
const createModelAsync = require('./model');

var _require = require('./swagger');

const parseAsync = _require.parseAsync;

var _require2 = require('./error');

const HttpError = _require2.HttpError,
      ParameterError = _require2.ParameterError;

const createWebSocketApplication = require('./ws');
const createSession = require('./session');
const createTestApplicationAsync = require('./supertest');

exports.createApplicationAsync = createApplicationAsync;
// backward compability
exports.ResponseError = exports.HttpError = HttpError;
exports.ParameterError = ParameterError;
exports.translate = exports.utility = utility;
exports.parseAsync = parseAsync;
exports.log = exports.logger = logger;
exports.rc = rc;
exports.wrapRoute = utility.wrapRoute;
exports.wrapMiddleware = utility.wrapMiddleware;
exports.DataTypes = Sequelize.DataTypes;
exports.QueryTypes = Sequelize.QueryTypes;
exports.getRestMiddleware = getRestMiddleware;

function getRestMiddleware() {
  var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  let api = _ref3.api,
      middlewares = _ref3.middlewares,
      swagger = _ref3.swagger,
      errorHandler = _ref3.errorHandler;

  const router = Router();

  if (!api || !middlewares) {
    load(swagger).then((_ref4) => {
      let api = _ref4.api,
          middlewares = _ref4.middlewares;

      router.use(getRestMiddleware({ api, middlewares, errorHandler }));
    }).catch(err => {
      console.log(err.stack); // eslint-disable-line
      process.exit(1);
    });
    return router;
  }

  //
  // Response pipeline
  //

  const pipeline = Pipeline();
  pipeline.use(mws.debug.postHandler());
  // remove response `null` fields
  pipeline.use(mws.validation.response());
  pipeline.use(mws.debug.response());
  router.use(pipeline.createMiddleware());

  //
  // Request pipeline
  //

  router.use((req, res, next) => {
    req.state = {
      version: api.definition.info.version
    };
    const operation = api.getOperation(req);
    req.swagger = {
      operation: operation
    };
    next();
  });
  // binding request context
  router.use(mws.context(rc.target + '/context'));
  // setup swagger documentation
  router.use(mws.docs(api.definition));
  router.use(mws.switch());
  // request & response logging
  router.use(mws.debug.request());
  // request validation
  router.use(mws.validation.request());
  // security handler
  router.use(mws.security(api, middlewares));
  // dynamic swagger defined route
  router.use(mws.debug.preHandler());
  router.use(api.basePathPrefix, mws.api(api));

  // 404
  router.use(api.basePathPrefix, () => {
    throw new HttpError('NotFound');
  });

  // error handling
  router.use(api.basePathPrefix, mws.errorHandler(api, errorHandler));

  return router;
}

process.on('unhandledRejection', function (reason, p) {
  let err = new Error('Unhandled Promise rejection');
  err.reason = reason;
  err.promise = p;
  logger.error('Unhandled Rejection at: Promise ', p, ' reason: ', reason);
  logger.error(reason.message, reason.stack);
  throw err;
});