'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

const express = require('express');
const ioredis = require('ioredis');
const ioSession = require('socket.io-express-session');
const ioRedis = require('socket.io-redis');
const MockReq = require('mock-req');
const _ = require('lodash');

const MockRes = require('./mock-res');
const rc = require('./rc');
const createSession = require('./session');
const logger = require('./logger');

function createWebSocketApplication(server, api, options) {
  if (!options) {
    return;
  }

  const io = require('socket.io')(server, { path: options.path });
  const wsapp = express();
  wsapp.io = io;
  wsapp.extendMiddleware = extendMiddleware;

  if (options.session) {
    if (options.session.redis) {
      io.adapter(ioRedis({
        key: `${rc.name}:socket.io`,
        pubClient: ioredis(options.session.redis),
        subClient: ioredis(options.session.redis),
        subEvent: 'messageBuffer'
      }));
    }
    io.use(ioSession(createSession(options.session)));
  }

  io.on('connection', socket => {
    const session = socket.handshake.session;
    // join sid and userId to allow send message to particular socket
    socket.join(`session ${session.id}`);
    // TODO: custom userId field?
    socket.join(`user ${session.userId}`);

    socket.on('req', message => {
      var _message = _slicedToArray(message, 2);

      let envelope = _message[0];
      var _message$ = _message[1],
          _message$$method = _message$.method;
      let method = _message$$method === undefined ? 'GET' : _message$$method,
          url = _message$.url;
      var _message$$headers = _message$.headers;
      let headers = _message$$headers === undefined ? {} : _message$$headers,
          query = _message$.query,
          body = _message$.body;

      headers = _.assign({ 'content-type': 'application/json' }, socket.handshake.headers, headers);
      const req = new MockReq({
        method,
        url,
        headers,
        // arbitrary properties:
        session,
        ip: socket.handshake.headers['x-forwarded-for'] || socket.handshake.address,
        query,
        body,
        io,
        socket
      });

      const res = MockRes(req, envelope, api.basePathPrefix);

      wsapp(req, res, err => {
        if (err) {
          return logger.error(err.stack);
        }

        logger.warn('WebSocket no matching handler');
      });
    });

    // NOTE: we cant listen to `join` event because normally sensitive message will broadcasted and `join` action
    // should have some sort of authentications.
  });

  // bind websocket similar api to request and response object
  function extendMiddleware(req, res, next) {
    req.io = io;
    res.join = function (room) {
      return this;
    };
    res.broadcast = function (body, room) {
      const res = {
        status: this.statusCode,
        statusMessage: this.statusMessage,
        headers: this.headers,
        body
      };
      req.io.to(room).emit(`${req.method} ${api.basePathPrefix + req.path}`, res);
      return this;
    };
    next();
  }

  return wsapp;
}

module.exports = createWebSocketApplication;